#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <descrip.h>

#include "vbisam.h"

#include "vms_vbisam.h"

#ifdef API21
#define GLBVAR_PREFIX vb_get_rtd()->
#define S (signed char *)
#else
#define GLBVAR_PREFIX
#define S
#endif

static char *in_cstr(struct dsc$descriptor *s)
{
    int slen, ix;
    char *sptr, *res;
    if(s->dsc$b_class == DSC$K_CLASS_S || s->dsc$b_class == DSC$K_CLASS_D)
    {
        slen = s->dsc$w_length;
        sptr = s->dsc$a_pointer;
    }
    else if(s->dsc$b_class == DSC$K_CLASS_VS)
    {
        slen = *((short int *)s->dsc$a_pointer);
        sptr = s->dsc$a_pointer + sizeof(short int);
    }
    else
    {
        fprintf(stderr, "Unsupported descriptor class: %d\n", s->dsc$b_class);
        exit(44);
    }
    res = malloc(slen + 1);
    strncpy(res, sptr, slen);
    res[slen] = 0;
    /* trim traling whitespace */
    ix = slen - 1;
    while(ix >= 0 && (res[ix] == ' ' || res[ix] == 0))
    {
        res[ix] = 0;
        ix--;
    }
    return res;
}

int db_create(struct dsc$descriptor *dbnam, int * reclen, int *keystart, int *keylen)
{
    int db;
    char *dbnam2;
    struct keydesc key;
    dbnam2 = in_cstr(dbnam);
    key.k_flags = 0;
    key.k_nparts = 1;
    key.k_part[0].kp_start = *keystart;
    key.k_part[0].kp_leng = *keylen;
    key.k_part[0].kp_type = 0;
    key.k_len = *keylen;
    key.k_rootnode = 0;
    db = isbuild(S dbnam2, *reclen, &key, ISINOUT + ISFIXLEN);
    free(dbnam2);
    return db;
}

int db_open(struct dsc$descriptor *dbnam)
{
    int db;
    char *dbnam2;
    dbnam2 = in_cstr(dbnam);
    db = isopen(S dbnam2, ISINOUT + ISFIXLEN);
    free(dbnam2);
    return db;
}

int db_put(int *db, char *buf)
{
    return iswrite(*db, S buf);
}

int db_get(int *db, char *buf, int *buflen)
{
    *buflen = GLBVAR_PREFIX isreclen;
    return isread(*db, S buf, ISEQUAL);
}

int db_delete(int *db, char *buf)
{
    return isdelete(*db, S buf);
}

int db_cursor_open(int *db)
{
    return *db;
}

int db_cursor_find(int *it, char *buf, int *buflen)
{
    *buflen = GLBVAR_PREFIX isreclen;
    return isread(*it, S buf, ISGTEQ); 
}

int db_cursor_next(int *it, char *buf, int *buflen)
{
    *buflen = GLBVAR_PREFIX isreclen;
    return isread(*it, S buf, ISNEXT);
}

void db_close(int *db)
{
    isclose(*db);
}

